"""Only required to hold the automatically generated version for the "core" extras"""

import typing as ty
import types
import inspect
from ._version import __version__


def check_optional_dependency(module: ty.Optional[types.ModuleType]) -> None:
    if module is None:
        frame = inspect.currentframe()
        while frame:
            # Find the frame where the decorated_extra method was called
            if frame.f_code.co_name == "decorated_extra":
                extras_module = frame.f_locals["extras"][0].pkg.split(".")[-1]
                break
            frame = frame.f_back
        raise ImportError(
            f"The optional dependencies are not installed for '{extras_module}', please include when "
            f"installing fileformats-extras, e.g. `pip install 'fileformats-extras[{extras_module}]'`"
        )


__all__ = ["__version__"]



import typing as ty


# import numpy  # noqa: F401
# import typing  # noqa: F401
from fileformats.extras.core import check_optional_dependency
from fileformats.core import FileSet, extra_implementation
from fileformats.image.raster import RasterImage, DataArrayType

try:
    import imageio  # noqa: F401
except ImportError:
    imageio = None  # type: ignore


@extra_implementation(FileSet.load)
def read_raster_data(image: RasterImage, **kwargs: ty.Any) -> DataArrayType:
    check_optional_dependency(imageio)

    return imageio.imread(image.fspath)  # type: ignore


@extra_implementation(FileSet.save)
def write_raster_data(
    image: RasterImage, data: DataArrayType, **kwargs: ty.Any
) -> None:
    check_optional_dependency(imageio)

    imageio.imwrite(image.fspath, data, **kwargs)



from fileformats.testing import MyFormat
import pytest


def test_check_optional_dependency_fail():

    with pytest.raises(
        ImportError, match="The optional dependencies are not installed for 'testing'"
    ):
        MyFormat.sample().dummy_extra()



from pathlib import Path

import attrs
import pytest
from pydra.compose import python, shell

from conftest import write_test_file
from fileformats.core import converter
from fileformats.core.exceptions import FormatConversionError
from fileformats.generic import File, FileSet
from fileformats.testing import (
    AnotherConcreteClass,
    Bar,
    Baz,
    ConcreteClass,
    ConvertibleToFile,
    Foo,
    Qux,
)


@converter
@python.define(outputs={"out_file": Bar})  # type: ignore[misc]
def FooBarConverter(in_file: Foo):
    return Bar(write_test_file(Path.cwd() / "bar.bar", in_file.raw_contents))


@converter(out_file="out")
@python.define(outputs={"out": Bar})  # type: ignore[misc]
def BazBarConverter(in_file: Baz):
    assert in_file
    return Bar(write_test_file(Path.cwd() / "bar.bar", in_file.raw_contents))


@converter(source_format=Foo, target_format=Qux)
@shell.define
class FooQuxConverter(shell.Task["FooQuxConverter.Outputs"]):

    in_file: File = shell.arg(help="the input file", argstr="")
    executable = "cp"

    class Outputs(shell.Outputs):
        out_file: File = shell.outarg(
            help="output file",
            argstr="",
            position=-1,
            path_template="out.qux",
        )


def test_get_converter_functask(work_dir):

    fspath = work_dir / "test.foo"
    write_test_file(fspath)
    assert attrs.asdict(Bar.get_converter(Foo).task) == attrs.asdict(FooBarConverter())


def test_get_converter_shellcmd(work_dir):

    fspath = work_dir / "test.foo"
    write_test_file(fspath)
    assert attrs.asdict(Qux.get_converter(Foo).task) == attrs.asdict(FooQuxConverter())


def test_get_converter_fail(work_dir):

    fspath = work_dir / "test.foo"
    write_test_file(fspath)
    with pytest.raises(FormatConversionError):
        Baz.get_converter(Foo)


def test_convert_functask(work_dir):

    fspath = work_dir / "test.foo"
    write_test_file(fspath)
    foo = Foo(fspath)
    bar = Bar.convert(foo)
    assert type(bar) is Bar
    assert bar.raw_contents == foo.raw_contents


def test_convert_shellcmd(work_dir):

    fspath = work_dir / "test.foo"
    write_test_file(fspath)
    foo = Foo(fspath)
    qux = Qux.convert(foo)
    assert type(qux) is Qux
    assert qux.raw_contents == foo.raw_contents


def test_convert_mapped_conversion(work_dir):

    fspath = work_dir / "test.baz"
    write_test_file(fspath)
    baz = Baz(fspath)
    bar = Bar.convert(baz)
    assert type(bar) is Bar
    assert bar.raw_contents == baz.raw_contents


@pytest.mark.parametrize(
    ["klass", "convertible_from"],
    [
        [Bar, Bar | Baz | Foo],
        [Qux, Foo | Qux],
        [Foo, Foo],
        [Baz, Baz],
        [
            ConvertibleToFile,
            AnotherConcreteClass | ConcreteClass | ConvertibleToFile,
        ],
    ],
)
def test_convertible_from(klass: type[FileSet], convertible_from: type[FileSet]):
    assert klass.convertible_from() == convertible_from



import typing as ty

try:
    import docx.document
except ImportError:
    docx = None  # type: ignore[assignment]

from fileformats.core import FileSet, extra_implementation
from fileformats.extras.core import check_optional_dependency
from fileformats.vendor.openxmlformats_officedocument.application import (
    Wordprocessingml_Document as MswordX,
)


@extra_implementation(FileSet.load)
def load_docx(doc: MswordX, **kwargs: ty.Any) -> "docx.document.Document":
    check_optional_dependency(docx)
    return docx.Document(str(doc))  # type: ignore[no-any-return]


@extra_implementation(FileSet.save)
def save_docx(doc: MswordX, data: "docx.document.Document", **kwargs: ty.Any) -> None:
    check_optional_dependency(docx)
    if not isinstance(data, docx.document.Document):
        raise TypeError(f"Expected a 'docx.document.Document' object, got {type(data)}")
    data.save(str(doc))



from pydra.compose import python

from fileformats.core import converter, extra_implementation
from fileformats.extras.core import check_optional_dependency
from fileformats.testing import AbstractFile, ConvertibleToFile, EncodedText, MyFormat
from fileformats.text import TextFile

dummy_import = None


@converter  # pyright: ignore[reportArgumentType]
@python.define(outputs=["out_file"])  # type: ignore[misc]
def ConvertibleToConverter(in_file: AbstractFile) -> ConvertibleToFile:
    return ConvertibleToFile.sample()


@converter  # pyright: ignore[reportArgumentType]
@python.define(outputs=["out_file"])  # type: ignore[misc]
def EncodedFromTextConverter(in_file: TextFile, shift: int = 1) -> EncodedText:
    contents = in_file.read_text()
    # Encode by shifting ASCII codes forward by 1
    encoded_contents = "".join(chr(ord(c) + shift) for c in contents)
    out_file = EncodedText.sample()
    out_file.write_text(encoded_contents)
    return out_file


@converter  # pyright: ignore[reportArgumentType]
@python.define(outputs=["out_file"])  # type: ignore[misc]
def EncodedToTextConverter(in_file: EncodedText, shift: int = 1) -> TextFile:
    contents = in_file.read_text()
    # Decode by shifting ASCII codes back by 1
    decoded_contents = "".join(chr(ord(c) - shift) for c in contents)
    out_file = TextFile.sample()
    out_file.write_text(decoded_contents)
    return out_file


@extra_implementation(MyFormat.dummy_extra)
def my_format_dummy_extra(my_format: MyFormat) -> int:
    check_optional_dependency(dummy_import)
    return 42
